@@ -97,12 +97,10 @@ struct hb_work_private_s
 
     int                  async_depth;
     int                  max_async_depth;
-    int                  max_buffer_size;
 
     // if encode-only, system memory used
     int                  is_sys_mem;
     mfxSession           mfx_session;
-    struct SwsContext  * sws_context_to_nv12;
 
     // whether to expect input from VPP or from QSV decode
     int                  is_vpp_present;
@@ -157,6 +155,232 @@ static int64_t get_frame_duration(hb_work_private_t *pv, hb_buffer_t *buf)
     return pv->frame_duration[i];
 }
 
+static inline int64_t rescale(hb_rational_t q, int b)
+{
+    return av_rescale(q.num, b, q.den);
+}
+
+static const char* hyper_encode_name(const int hyper_encode_mode)
+{
+    switch (hyper_encode_mode)
+    {
+        case MFX_HYPERMODE_OFF:
+            return "(HyperEncode Off)";
+
+        case MFX_HYPERMODE_ON:
+            return "(HyperEncode On)";
+
+        case MFX_HYPERMODE_ADAPTIVE:
+            return "(HyperEncode Adaptive)";
+
+        default:
+            return NULL;
+    }
+}
+
+static int log_encoder_params(const hb_work_private_t *pv, const mfxVideoParam *videoParam)
+{
+    const mfxExtCodingOption  *option1 = NULL;
+    const mfxExtCodingOption2 *option2 = NULL;
+    const mfxExtHyperModeParam *extHyperModeOption = NULL;
+
+    for (int i = 0; i < videoParam->NumExtParam; i++)
+    {
+        mfxExtCodingOption *option = (mfxExtCodingOption*)videoParam->ExtParam[i];
+        if (option->Header.BufferId == MFX_EXTBUFF_CODING_OPTION)
+        {
+            option1 = (mfxExtCodingOption*)videoParam->ExtParam[i];
+        }
+        else if (option->Header.BufferId == MFX_EXTBUFF_CODING_OPTION2)
+        {
+            option2 = (mfxExtCodingOption2*)videoParam->ExtParam[i];
+        }
+        else if (option->Header.BufferId == MFX_EXTBUFF_HYPER_MODE_PARAM)
+        {
+            extHyperModeOption = (mfxExtHyperModeParam*)videoParam->ExtParam[i];
+        }
+        else if (option->Header.BufferId != MFX_EXTBUFF_VIDEO_SIGNAL_INFO &&
+                 option->Header.BufferId != MFX_EXTBUFF_CHROMA_LOC_INFO &&
+                 option->Header.BufferId != MFX_EXTBUFF_MASTERING_DISPLAY_COLOUR_VOLUME &&
+                 option->Header.BufferId != MFX_EXTBUFF_CONTENT_LIGHT_LEVEL_INFO)
+        {
+            hb_log("Unknown Header.BufferId=%d", option->Header.BufferId);
+        }
+    }
+
+    // log code path and main output settings
+    hb_log("encqsvInit: using%s%s%s path",
+           pv->is_sys_mem ? " encode-only" : " full QSV",
+           videoParam->mfx.LowPower == MFX_CODINGOPTION_ON ? " (LowPower)" : "",
+           extHyperModeOption != NULL ? hyper_encode_name(extHyperModeOption->Mode) : "");
+    hb_log("encqsvInit: %s %s profile @ level %s",
+           hb_qsv_codec_name  (videoParam->mfx.CodecId),
+           hb_qsv_profile_name(videoParam->mfx.CodecId, videoParam->mfx.CodecProfile),
+           hb_qsv_level_name  (videoParam->mfx.CodecId, videoParam->mfx.CodecLevel));
+    hb_log("encqsvInit: TargetUsage %"PRIu16" AsyncDepth %"PRIu16"",
+           videoParam->mfx.TargetUsage, videoParam->AsyncDepth);
+    hb_log("encqsvInit: GopRefDist %"PRIu16" GopPicSize %"PRIu16" NumRefFrame %"PRIu16" IdrInterval %"PRIu16"",
+           videoParam->mfx.GopRefDist, videoParam->mfx.GopPicSize, videoParam->mfx.NumRefFrame, videoParam->mfx.IdrInterval);
+
+    if (pv->qsv_info->capabilities & HB_QSV_CAP_B_REF_PYRAMID)
+    {
+        hb_log("encqsvInit: BFramesMax %d BRefType %s",
+               videoParam->mfx.GopRefDist > 1 ?
+               videoParam->mfx.GopRefDist - 1 : 0,
+               pv->param.gop.b_pyramid ? "pyramid" : "off");
+    }
+    else
+    {
+        hb_log("encqsvInit: BFramesMax %d",
+               videoParam->mfx.GopRefDist > 1 ?
+               videoParam->mfx.GopRefDist - 1 : 0);
+    }
+
+    if (option2 && (option2->AdaptiveI != MFX_CODINGOPTION_OFF ||
+        option2->AdaptiveB != MFX_CODINGOPTION_OFF))
+    {
+        if (videoParam->mfx.GopRefDist > 1)
+        {
+            hb_log("encqsvInit: AdaptiveI %s AdaptiveB %s",
+                hb_qsv_codingoption_get_name(option2->AdaptiveI),
+                hb_qsv_codingoption_get_name(option2->AdaptiveB));
+        }
+        else
+        {
+            hb_log("encqsvInit: AdaptiveI %s",
+                hb_qsv_codingoption_get_name(option2->AdaptiveI));
+        }
+    }
+
+    if (videoParam->mfx.RateControlMethod == MFX_RATECONTROL_CQP)
+    {
+        char qpi[7], qpp[9], qpb[9];
+        snprintf(qpi, sizeof(qpi),  "QPI %"PRIu16"", videoParam->mfx.QPI);
+        snprintf(qpp, sizeof(qpp), " QPP %"PRIu16"", videoParam->mfx.QPP);
+        snprintf(qpb, sizeof(qpb), " QPB %"PRIu16"", videoParam->mfx.QPB);
+        hb_log("encqsvInit: RateControlMethod CQP with %s%s%s", qpi,
+               videoParam->mfx.GopPicSize > 1 ? qpp : "",
+               videoParam->mfx.GopRefDist > 1 ? qpb : "");
+    }
+    else
+    {
+        switch (videoParam->mfx.RateControlMethod)
+        {
+            case MFX_RATECONTROL_LA:
+                hb_log("encqsvInit: RateControlMethod LA TargetKbps %"PRIu16" BRCParamMultiplier %"PRIu16" LookAheadDepth %"PRIu16"",
+                       videoParam->mfx.TargetKbps, videoParam->mfx.BRCParamMultiplier, (option2 != NULL) ? option2->LookAheadDepth : 0);
+                break;
+            case MFX_RATECONTROL_LA_ICQ:
+                hb_log("encqsvInit: RateControlMethod LA_ICQ ICQQuality %"PRIu16" LookAheadDepth %"PRIu16"",
+                       videoParam->mfx.ICQQuality, (option2 != NULL) ? option2->LookAheadDepth : 0);
+                break;
+            case MFX_RATECONTROL_ICQ:
+                hb_log("encqsvInit: RateControlMethod ICQ ICQQuality %"PRIu16"",
+                       videoParam->mfx.ICQQuality);
+                break;
+            case MFX_RATECONTROL_CBR:
+            case MFX_RATECONTROL_VBR:
+                hb_log("encqsvInit: RateControlMethod %s TargetKbps %"PRIu16" MaxKbps %"PRIu16" BufferSizeInKB %"PRIu16" InitialDelayInKB %"PRIu16" BRCParamMultiplier %"PRIu16"",
+                       videoParam->mfx.RateControlMethod == MFX_RATECONTROL_CBR ? "CBR" : "VBR",
+                       videoParam->mfx.TargetKbps,     videoParam->mfx.MaxKbps,
+                       videoParam->mfx.BufferSizeInKB, videoParam->mfx.InitialDelayInKB, videoParam->mfx.BRCParamMultiplier);
+                break;
+            default:
+                hb_log("encqsvInit: invalid rate control method %"PRIu16"",
+                       videoParam->mfx.RateControlMethod);
+                return -1;
+        }
+    }
+
+    if (option2 && (videoParam->mfx.RateControlMethod == MFX_RATECONTROL_LA ||
+        videoParam->mfx.RateControlMethod == MFX_RATECONTROL_LA_ICQ))
+    {
+        switch (option2->LookAheadDS)
+        {
+            case MFX_LOOKAHEAD_DS_UNKNOWN:
+                hb_log("encqsvInit: LookAheadDS unknown (auto)");
+                break;
+            case MFX_LOOKAHEAD_DS_OFF: // default
+                break;
+            case MFX_LOOKAHEAD_DS_2x:
+                hb_log("encqsvInit: LookAheadDS 2x");
+                break;
+            case MFX_LOOKAHEAD_DS_4x:
+                hb_log("encqsvInit: LookAheadDS 4x");
+                break;
+            default:
+                hb_log("encqsvInit: invalid LookAheadDS value 0x%"PRIx16"",
+                       option2->LookAheadDS);
+                break;
+        }
+    }
+
+    switch (videoParam->mfx.FrameInfo.PicStruct)
+    {
+        case MFX_PICSTRUCT_PROGRESSIVE: // default
+            break;
+        case MFX_PICSTRUCT_FIELD_TFF:
+            hb_log("encqsvInit: PicStruct top field first");
+            break;
+        case MFX_PICSTRUCT_FIELD_BFF:
+            hb_log("encqsvInit: PicStruct bottom field first");
+            break;
+        default:
+            hb_error("encqsvInit: invalid PicStruct value 0x%"PRIx16"",
+                     videoParam->mfx.FrameInfo.PicStruct);
+            return -1;
+    }
+
+    if (videoParam->mfx.CodecId == MFX_CODEC_AVC)
+    {
+        if (option1 && (option1->CAVLC != MFX_CODINGOPTION_OFF))
+        {
+            hb_log("encqsvInit: CAVLC %s",
+                hb_qsv_codingoption_get_name(option1->CAVLC));
+        }
+    }
+
+    if (option2 && (option2->ExtBRC != MFX_CODINGOPTION_OFF))
+    {
+        hb_log("encqsvInit: ExtBRC %s",
+            hb_qsv_codingoption_get_name(option2->ExtBRC));
+    }
+
+    if (option2 && (option2->MBBRC != MFX_CODINGOPTION_OFF))
+    {
+        hb_log("encqsvInit: MBBRC %s",
+            hb_qsv_codingoption_get_name(option2->MBBRC));
+    }
+
+    if (option2)
+    {
+        switch (option2->Trellis)
+        {
+            case MFX_TRELLIS_OFF: // default
+                break;
+            case MFX_TRELLIS_UNKNOWN:
+                hb_log("encqsvInit: Trellis unknown (auto)");
+                break;
+            default:
+                hb_log("encqsvInit: Trellis on (%s%s%s)",
+                       (option2->Trellis & MFX_TRELLIS_I) ? "I" : "",
+                       (option2->Trellis & MFX_TRELLIS_P) &&
+                       (videoParam->mfx.GopPicSize > 1)    ? "P" : "",
+                       (option2->Trellis & MFX_TRELLIS_B) &&
+                       (videoParam->mfx.GopRefDist > 1)    ? "B" : "");
+                break;
+        }
+    }
+
+    if (option2 && (option2->RepeatPPS != MFX_CODINGOPTION_OFF))
+    {
+        hb_log("encqsvInit: RepeatPPS %s",
+            hb_qsv_codingoption_get_name(option2->RepeatPPS));
+    }
+
+    return 0;
+}
+
 static void qsv_handle_breftype(hb_work_private_t *pv)
 {
     /*
@@ -288,13 +512,12 @@ unsupported:
     pv->param.codingOption2.BRefType = MFX_B_REF_OFF;
 }
 
-static int qsv_hevc_make_header(hb_work_object_t *w, mfxSession session)
+static int qsv_hevc_make_header(hb_work_object_t *w, mfxSession session, const mfxVideoParam *videoParam)
 {
     size_t len;
     int ret = 0;
     uint8_t *buf, *end;
     mfxBitstream bitstream;
-    hb_buffer_t *bitstream_buf;
     mfxStatus status;
     mfxSyncPoint syncPoint;
     mfxFrameSurface1 frameSurface1;
@@ -304,26 +527,25 @@ static int qsv_hevc_make_header(hb_work_object_t *w, mfxSession session)
     memset(&syncPoint,     0, sizeof(mfxSyncPoint));
     memset(&frameSurface1, 0, sizeof(mfxFrameSurface1));
 
-    /* The bitstream buffer should be able to hold any encoded frame */
-    bitstream_buf = hb_video_buffer_init(pv->job->width, pv->job->height);
-    if (bitstream_buf == NULL)
+    if (videoParam == NULL)
     {
-        hb_log("qsv_hevc_make_header: hb_buffer_init failed");
+        hb_log("qsv_hevc_make_header: videoParam is NULL");
+        ret = -1;
+        goto end;
+    }
+    /* The bitstream buffer should be able to hold any encoded frame */
+    size_t buf_max_size = videoParam->mfx.BufferSizeInKB * 1000 * ( 0 == videoParam->mfx.BRCParamMultiplier ? 1 : videoParam->mfx.BRCParamMultiplier);
+    bitstream.Data      = av_mallocz(sizeof(uint8_t) * buf_max_size);
+    bitstream.MaxLength = buf_max_size;
+    if (bitstream.Data == NULL)
+    {
+        hb_log("qsv_hevc_make_header: bitstream.Data allocation failed");
         ret = -1;
         goto end;
     }
 
-    /* need more space for 10bits */
-    if (pv->param.videoParam->mfx.FrameInfo.FourCC == MFX_FOURCC_P010 ||
-        pv->param.videoParam->mfx.CodecId == MFX_CODEC_HEVC)
-    {
-         hb_buffer_realloc(bitstream_buf,bitstream_buf->size*2);
-    }
-    int bpp12 = (pv->param.videoParam->mfx.FrameInfo.FourCC == MFX_FOURCC_P010) ? 6 : 3;
-    bitstream.Data      = bitstream_buf->data;
-    bitstream.MaxLength = bitstream_buf->alloc;
-
     /* We only need to encode one frame, so we only need one surface */
+    int bpp12                = (pv->param.videoParam->mfx.FrameInfo.FourCC == MFX_FOURCC_P010) ? 6 : 3;
     mfxU16 Height            = pv->param.videoParam->mfx.FrameInfo.Height;
     mfxU16 Width             = pv->param.videoParam->mfx.FrameInfo.Width;
     frameSurface1.Info       = pv->param.videoParam->mfx.FrameInfo;
@@ -346,6 +568,11 @@ static int qsv_hevc_make_header(hb_work_object_t *w, mfxSession session)
         if (status < MFX_ERR_NONE)
         {
             hb_log("qsv_hevc_make_header: MFXVideoENCODE_EncodeFrameAsync failed (%d)", status);
+            status = log_encoder_params(pv, pv->param.videoParam);
+            if (status < 0)
+            {
+                hb_error("qsv_hevc_make_header: log_encoder_params failed (%d)", status);
+            }
             ret = -1;
             goto end;
         }
@@ -454,7 +681,8 @@ static int qsv_hevc_make_header(hb_work_object_t *w, mfxSession session)
     }
 
 end:
-    hb_buffer_close(&bitstream_buf);
+    if (bitstream.Data)
+        av_free(bitstream.Data);
     av_free(frameSurface1.Data.Y);
     return ret;
 }
@@ -490,7 +718,7 @@ AVBufferRef *hb_qsv_create_mids(AVBufferRef *hw_frames_ref)
     if (!hw_frames_ref1)
         return NULL;
 
-    mids = av_mallocz_array(nb_surfaces, sizeof(*mids));
+    mids = av_calloc(nb_surfaces, sizeof(*mids));
     if (!mids) {
         av_buffer_unref(&hw_frames_ref1);
         return NULL;
@@ -525,7 +753,7 @@ static int qsv_setup_mids(mfxFrameAllocResponse *resp, AVBufferRef *hw_frames_re
     // the allocated size of the array is two larger than the number of
     // surfaces, we store the references to the frames context and the
     // QSVMid array there
-    resp->mids = av_mallocz_array(nb_surfaces + 2, sizeof(*resp->mids));
+    resp->mids = av_calloc(nb_surfaces + 2, sizeof(*resp->mids));
     if (!resp->mids)
         return AVERROR(ENOMEM);
 
@@ -846,32 +1074,9 @@ int qsv_enc_init(hb_work_private_t *pv)
             return 2;
         }
     }
-    else
-    {
-        if (pv->param.videoParam->mfx.CodecProfile == MFX_PROFILE_HEVC_MAIN10)
-        {
-            pv->sws_context_to_nv12 = hb_sws_get_context(
-                                        job->width, job->height,
-                                        job->output_pix_fmt, job->color_range,
-                                        job->width, job->height,
-                                        AV_PIX_FMT_P010LE, job->color_range,
-                                        SWS_LANCZOS|SWS_ACCURATE_RND,
-                                        SWS_CS_DEFAULT);
-        }
-        else
-        {
-            pv->sws_context_to_nv12 = hb_sws_get_context(
-                                        job->width, job->height,
-                                        job->output_pix_fmt, job->color_range,
-                                        job->width, job->height,
-                                        AV_PIX_FMT_NV12, job->color_range,
-                                        SWS_LANCZOS|SWS_ACCURATE_RND,
-                                        SWS_CS_DEFAULT);
-        }
-    }
 
     // allocate tasks
-    qsv_encode->p_buf_max_size = pv->max_buffer_size * 1000;
+    qsv_encode->p_buf_max_size = pv->param.videoParam->mfx.BufferSizeInKB * 1000 * ( 0 == pv->param.videoParam->mfx.BRCParamMultiplier ? 1 : pv->param.videoParam->mfx.BRCParamMultiplier);
     qsv_encode->tasks          = hb_qsv_list_init(HAVE_THREADS);
     for (i = 0; i < pv->max_async_depth; i++)
     {
@@ -933,7 +1138,7 @@ int qsv_enc_init(hb_work_private_t *pv)
         }
 
         /* should have 15bpp/AV_PIX_FMT_YUV420P10LE (almost x2) instead of 12bpp/AV_PIX_FMT_NV12 */
-        int bpp12 = (pv->param.videoParam->mfx.CodecProfile == MFX_PROFILE_HEVC_MAIN10) ? 6 : 3;
+        int bpp12 = (pv->param.videoParam->mfx.CodecProfile == (MFX_PROFILE_HEVC_MAIN10) || (job->vcodec == HB_VCODEC_QSV_AV1_10BIT)) ? 6 : 3;
         for (i = 0; i < qsv_encode->surface_num; i++)
         {
             mfxFrameSurface1 *surface = av_mallocz(sizeof(mfxFrameSurface1));
@@ -1002,199 +1207,6 @@ int qsv_enc_init(hb_work_private_t *pv)
     return 0;
 }
 
-static int log_encoder_params(const hb_work_private_t *pv, const mfxVideoParam *videoParam)
-{
-    const mfxExtCodingOption  *option1 = NULL;
-    const mfxExtCodingOption2 *option2 = NULL;
-    const mfxExtVideoSignalInfo *extVideoSignalInfo = NULL;
-
-    // log code path and main output settings
-    hb_log("encqsvInit: using %s%s path",
-           pv->is_sys_mem ? "encode-only" : "full QSV",
-           videoParam->mfx.LowPower == MFX_CODINGOPTION_ON ? " (LowPower)" : "" );
-    hb_log("encqsvInit: %s %s profile @ level %s",
-           hb_qsv_codec_name  (videoParam->mfx.CodecId),
-           hb_qsv_profile_name(videoParam->mfx.CodecId, videoParam->mfx.CodecProfile),
-           hb_qsv_level_name  (videoParam->mfx.CodecId, videoParam->mfx.CodecLevel));
-    hb_log("encqsvInit: TargetUsage %"PRIu16" AsyncDepth %"PRIu16"",
-           videoParam->mfx.TargetUsage, videoParam->AsyncDepth);
-    hb_log("encqsvInit: GopRefDist %"PRIu16" GopPicSize %"PRIu16" NumRefFrame %"PRIu16"",
-           videoParam->mfx.GopRefDist, videoParam->mfx.GopPicSize, videoParam->mfx.NumRefFrame);
-
-    if (pv->qsv_info->capabilities & HB_QSV_CAP_B_REF_PYRAMID)
-    {
-        hb_log("encqsvInit: BFramesMax %d BRefType %s",
-               videoParam->mfx.GopRefDist > 1 ?
-               videoParam->mfx.GopRefDist - 1 : 0,
-               pv->param.gop.b_pyramid ? "pyramid" : "off");
-    }
-    else
-    {
-        hb_log("encqsvInit: BFramesMax %d",
-               videoParam->mfx.GopRefDist > 1 ?
-               videoParam->mfx.GopRefDist - 1 : 0);
-    }
-
-    for (int i = 0; i < videoParam->NumExtParam; i++)
-    {
-        mfxExtCodingOption *option = (mfxExtCodingOption*)videoParam->ExtParam[i];
-        if (option->Header.BufferId == MFX_EXTBUFF_VIDEO_SIGNAL_INFO)
-        {
-            extVideoSignalInfo = (mfxExtVideoSignalInfo*)videoParam->ExtParam[i];
-        }
-        else if (option->Header.BufferId == MFX_EXTBUFF_CODING_OPTION)
-        {
-            option1 = (mfxExtCodingOption*)videoParam->ExtParam[i];
-        }
-        else if (option->Header.BufferId == MFX_EXTBUFF_CODING_OPTION2)
-        {
-            option2 = (mfxExtCodingOption2*)videoParam->ExtParam[i];
-        }
-        else
-        {
-            hb_log("Unknown Header.BufferId=%d", option->Header.BufferId);
-        }
-    }
-
-    if (option2 && (option2->AdaptiveI != MFX_CODINGOPTION_OFF ||
-        option2->AdaptiveB != MFX_CODINGOPTION_OFF))
-    {
-        if (videoParam->mfx.GopRefDist > 1)
-        {
-            hb_log("encqsvInit: AdaptiveI %s AdaptiveB %s",
-                hb_qsv_codingoption_get_name(option2->AdaptiveI),
-                hb_qsv_codingoption_get_name(option2->AdaptiveB));
-        }
-        else
-        {
-            hb_log("encqsvInit: AdaptiveI %s",
-                hb_qsv_codingoption_get_name(option2->AdaptiveI));
-        }
-    }
-
-    if (videoParam->mfx.RateControlMethod == MFX_RATECONTROL_CQP)
-    {
-        char qpi[7], qpp[9], qpb[9];
-        snprintf(qpi, sizeof(qpi),  "QPI %"PRIu16"", videoParam->mfx.QPI);
-        snprintf(qpp, sizeof(qpp), " QPP %"PRIu16"", videoParam->mfx.QPP);
-        snprintf(qpb, sizeof(qpb), " QPB %"PRIu16"", videoParam->mfx.QPB);
-        hb_log("encqsvInit: RateControlMethod CQP with %s%s%s", qpi,
-               videoParam->mfx.GopPicSize > 1 ? qpp : "",
-               videoParam->mfx.GopRefDist > 1 ? qpb : "");
-    }
-    else
-    {
-        switch (videoParam->mfx.RateControlMethod)
-        {
-            case MFX_RATECONTROL_LA:
-                hb_log("encqsvInit: RateControlMethod LA TargetKbps %"PRIu16" BRCParamMultiplier %"PRIu16" LookAheadDepth %"PRIu16"",
-                       videoParam->mfx.TargetKbps, videoParam->mfx.BRCParamMultiplier, (option2 != NULL) ? option2->LookAheadDepth : 0);
-                break;
-            case MFX_RATECONTROL_LA_ICQ:
-                hb_log("encqsvInit: RateControlMethod LA_ICQ ICQQuality %"PRIu16" LookAheadDepth %"PRIu16"",
-                       videoParam->mfx.ICQQuality, (option2 != NULL) ? option2->LookAheadDepth : 0);
-                break;
-            case MFX_RATECONTROL_ICQ:
-                hb_log("encqsvInit: RateControlMethod ICQ ICQQuality %"PRIu16"",
-                       videoParam->mfx.ICQQuality);
-                break;
-            case MFX_RATECONTROL_CBR:
-            case MFX_RATECONTROL_VBR:
-                hb_log("encqsvInit: RateControlMethod %s TargetKbps %"PRIu16" MaxKbps %"PRIu16" BufferSizeInKB %"PRIu16" InitialDelayInKB %"PRIu16" BRCParamMultiplier %"PRIu16"",
-                       videoParam->mfx.RateControlMethod == MFX_RATECONTROL_CBR ? "CBR" : "VBR",
-                       videoParam->mfx.TargetKbps,     videoParam->mfx.MaxKbps,
-                       videoParam->mfx.BufferSizeInKB, videoParam->mfx.InitialDelayInKB, videoParam->mfx.BRCParamMultiplier);
-                break;
-            default:
-                hb_log("encqsvInit: invalid rate control method %"PRIu16"",
-                       videoParam->mfx.RateControlMethod);
-                return -1;
-        }
-    }
-
-    if (option2 && (videoParam->mfx.RateControlMethod == MFX_RATECONTROL_LA ||
-        videoParam->mfx.RateControlMethod == MFX_RATECONTROL_LA_ICQ))
-    {
-        switch (option2->LookAheadDS)
-        {
-            case MFX_LOOKAHEAD_DS_UNKNOWN:
-                hb_log("encqsvInit: LookAheadDS unknown (auto)");
-                break;
-            case MFX_LOOKAHEAD_DS_OFF: // default
-                break;
-            case MFX_LOOKAHEAD_DS_2x:
-                hb_log("encqsvInit: LookAheadDS 2x");
-                break;
-            case MFX_LOOKAHEAD_DS_4x:
-                hb_log("encqsvInit: LookAheadDS 4x");
-                break;
-            default:
-                hb_log("encqsvInit: invalid LookAheadDS value 0x%"PRIx16"",
-                       option2->LookAheadDS);
-                break;
-        }
-    }
-
-    switch (videoParam->mfx.FrameInfo.PicStruct)
-    {
-        case MFX_PICSTRUCT_PROGRESSIVE: // default
-            break;
-        case MFX_PICSTRUCT_FIELD_TFF:
-            hb_log("encqsvInit: PicStruct top field first");
-            break;
-        case MFX_PICSTRUCT_FIELD_BFF:
-            hb_log("encqsvInit: PicStruct bottom field first");
-            break;
-        default:
-            hb_error("encqsvInit: invalid PicStruct value 0x%"PRIx16"",
-                     videoParam->mfx.FrameInfo.PicStruct);
-            return -1;
-    }
-
-    if (videoParam->mfx.CodecId == MFX_CODEC_AVC)
-    {
-        if (option1 && (option1->CAVLC != MFX_CODINGOPTION_OFF))
-        {
-            hb_log("encqsvInit: CAVLC %s",
-                hb_qsv_codingoption_get_name(option1->CAVLC));
-        }
-    }
-
-    if (option2 && (option2->ExtBRC != MFX_CODINGOPTION_OFF))
-    {
-        hb_log("encqsvInit: ExtBRC %s",
-            hb_qsv_codingoption_get_name(option2->ExtBRC));
-    }
-
-    if (option2 && (option2->MBBRC != MFX_CODINGOPTION_OFF))
-    {
-        hb_log("encqsvInit: MBBRC %s",
-            hb_qsv_codingoption_get_name(option2->MBBRC));
-    }
-
-    if (option2)
-    {
-        switch (option2->Trellis)
-        {
-            case MFX_TRELLIS_OFF: // default
-                break;
-            case MFX_TRELLIS_UNKNOWN:
-                hb_log("encqsvInit: Trellis unknown (auto)");
-                break;
-            default:
-                hb_log("encqsvInit: Trellis on (%s%s%s)",
-                       (option2->Trellis & MFX_TRELLIS_I) ? "I" : "",
-                       (option2->Trellis & MFX_TRELLIS_P) &&
-                       (videoParam->mfx.GopPicSize > 1)    ? "P" : "",
-                       (option2->Trellis & MFX_TRELLIS_B) &&
-                       (videoParam->mfx.GopRefDist > 1)    ? "B" : "");
-                break;
-        }
-    }
-
-    return 0;
-}
-
 /***********************************************************************
  * encqsvInit
  ***********************************************************************
@@ -1232,6 +1244,44 @@ int encqsvInit(hb_work_object_t *w, hb_job_t *job)
     pv->param.videoSignalInfo.MatrixCoefficients       = hb_output_color_matrix(job);
     pv->param.videoSignalInfo.ColourDescriptionPresent = 1;
 
+    if (job->chroma_location != AVCHROMA_LOC_UNSPECIFIED)
+    {
+        pv->param.chromaLocInfo.ChromaSampleLocTypeBottomField =
+        pv->param.chromaLocInfo.ChromaSampleLocTypeTopField = job->chroma_location - 1;
+        pv->param.chromaLocInfo.ChromaLocInfoPresentFlag = 1;
+    }
+
+    /* HDR10 Static metadata */
+    if (job->color_transfer == HB_COLR_TRA_SMPTEST2084)
+    {
+        const int masteringChromaDen = 50000;
+        const int masteringLumaDen = 10000;
+
+        /* Mastering display metadata */
+        if (job->mastering.has_primaries && job->mastering.has_luminance)
+        {
+            pv->param.masteringDisplayColourVolume.InsertPayloadToggle = MFX_PAYLOAD_IDR;
+            pv->param.masteringDisplayColourVolume.DisplayPrimariesX[0] = rescale(job->mastering.display_primaries[0][0], masteringChromaDen);
+            pv->param.masteringDisplayColourVolume.DisplayPrimariesY[0] = rescale(job->mastering.display_primaries[0][1], masteringChromaDen);
+            pv->param.masteringDisplayColourVolume.DisplayPrimariesX[1] = rescale(job->mastering.display_primaries[1][0], masteringChromaDen);
+            pv->param.masteringDisplayColourVolume.DisplayPrimariesY[1] = rescale(job->mastering.display_primaries[1][1], masteringChromaDen);
+            pv->param.masteringDisplayColourVolume.DisplayPrimariesX[2] = rescale(job->mastering.display_primaries[2][0], masteringChromaDen);
+            pv->param.masteringDisplayColourVolume.DisplayPrimariesY[2] = rescale(job->mastering.display_primaries[2][1], masteringChromaDen);
+            pv->param.masteringDisplayColourVolume.WhitePointX = rescale(job->mastering.white_point[0], masteringChromaDen);
+            pv->param.masteringDisplayColourVolume.WhitePointY = rescale(job->mastering.white_point[1], masteringChromaDen);
+            pv->param.masteringDisplayColourVolume.MaxDisplayMasteringLuminance = rescale(job->mastering.max_luminance, masteringLumaDen);
+            pv->param.masteringDisplayColourVolume.MinDisplayMasteringLuminance = rescale(job->mastering.min_luminance, masteringLumaDen);
+        }
+
+        /*  Content light level */
+        if (job->coll.max_cll && job->coll.max_fall)
+        {
+            pv->param.contentLightLevelInfo.InsertPayloadToggle = MFX_PAYLOAD_IDR;
+            pv->param.contentLightLevelInfo.MaxContentLightLevel  = job->coll.max_cll;
+            pv->param.contentLightLevelInfo.MaxPicAverageLightLevel = job->coll.max_fall;
+        }
+    }
+
     // parse user-specified encoder options, if present
     if (job->encoder_options != NULL && *job->encoder_options)
     {
@@ -1300,6 +1350,7 @@ int encqsvInit(hb_work_object_t *w, hb_job_t *job)
     switch (pv->qsv_info->codec_id)
     {
         case MFX_CODEC_HEVC:
+        case MFX_CODEC_AV1:
             job->qsv.enc_info.align_width  = HB_QSV_ALIGN32(job->width);
             job->qsv.enc_info.align_height = HB_QSV_ALIGN32(job->height);
             break;
@@ -1357,7 +1408,7 @@ int encqsvInit(hb_work_object_t *w, hb_job_t *job)
         return -1;
     }
 
-    if (pv->param.videoParam->mfx.CodecProfile == MFX_PROFILE_HEVC_MAIN10)
+    if ((pv->param.videoParam->mfx.CodecProfile == MFX_PROFILE_HEVC_MAIN10) || (job->vcodec == HB_VCODEC_QSV_AV1_10BIT))
     {
         pv->param.videoParam->mfx.FrameInfo.FourCC         = MFX_FOURCC_P010;
         pv->param.videoParam->mfx.FrameInfo.BitDepthLuma   = 10;
@@ -1453,6 +1504,8 @@ int encqsvInit(hb_work_object_t *w, hb_job_t *job)
     // set rate control parameters
     if (job->vquality > HB_INVALID_VIDEO_QUALITY)
     {
+        unsigned int upper_limit = 51;
+
         if (pv->param.rc.icq)
         {
             // introduced in API 1.8
@@ -1464,19 +1517,22 @@ int encqsvInit(hb_work_object_t *w, hb_job_t *job)
             {
                 pv->param.videoParam->mfx.RateControlMethod = MFX_RATECONTROL_ICQ;
             }
-            pv->param.videoParam->mfx.ICQQuality = HB_QSV_CLIP3(1, 51, job->vquality);
+            pv->param.videoParam->mfx.ICQQuality = HB_QSV_CLIP3(1, upper_limit, job->vquality);
         }
         else
         {
             // introduced in API 1.1
             // HEVC 10b has QP range as [-12;51]
             // with shift +12 needed to be in QSV's U16 range
-            unsigned int upper_limit = 51;
-
             if (pv->param.videoParam->mfx.CodecProfile == MFX_PROFILE_HEVC_MAIN10)
             {
                 upper_limit = 63;
             }
+            if (pv->param.videoParam->mfx.CodecId == MFX_CODEC_AV1)
+            {
+                upper_limit = 255;
+            }
+
             pv->param.videoParam->mfx.RateControlMethod = MFX_RATECONTROL_CQP;
             pv->param.videoParam->mfx.QPI = HB_QSV_CLIP3(0, upper_limit, job->vquality + pv->param.rc.cqp_offsets[0]);
             pv->param.videoParam->mfx.QPP = HB_QSV_CLIP3(0, upper_limit, job->vquality + pv->param.rc.cqp_offsets[1]);
@@ -1539,13 +1595,15 @@ int encqsvInit(hb_work_object_t *w, hb_job_t *job)
     // set the GOP structure
     if (pv->param.gop.gop_ref_dist < 0)
     {
-        if (pv->param.videoParam->mfx.RateControlMethod == MFX_RATECONTROL_CQP)
+        if ((hb_qsv_hardware_generation(hb_qsv_get_platform(hb_qsv_get_adapter_index())) >= QSV_G8) &&
+            (pv->param.videoParam->mfx.CodecId == MFX_CODEC_HEVC ||
+            pv->param.videoParam->mfx.CodecId == MFX_CODEC_AV1))
         {
-            pv->param.gop.gop_ref_dist = 4;
+            pv->param.gop.gop_ref_dist = 8;
         }
         else
         {
-            pv->param.gop.gop_ref_dist = 3;
+            pv->param.gop.gop_ref_dist = 4;
         }
     }
     pv->param.videoParam->mfx.GopRefDist = pv->param.gop.gop_ref_dist;
@@ -1554,19 +1612,24 @@ int encqsvInit(hb_work_object_t *w, hb_job_t *job)
     if (pv->param.gop.gop_pic_size < 0)
     {
         int rate = (double)job->orig_vrate.num / job->orig_vrate.den + 0.5;
-        if (pv->param.videoParam->mfx.RateControlMethod == MFX_RATECONTROL_CQP)
-        {
-            // ensure B-pyramid is enabled for CQP on Haswell
-            pv->param.gop.gop_pic_size = 32;
-        }
-        else
-        {
-            // set the keyframe interval based on the framerate
-            pv->param.gop.gop_pic_size = rate;
-        }
+        // set the keyframe interval based on the framerate
+        pv->param.gop.gop_pic_size = rate;
     }
     pv->param.videoParam->mfx.GopPicSize = pv->param.gop.gop_pic_size;
 
+    // set the Hyper Encode structure
+    if (pv->param.hyperEncodeParam.Mode != MFX_HYPERMODE_OFF)
+    {
+        if (pv->param.videoParam->mfx.CodecId == MFX_CODEC_HEVC)
+        {
+            pv->param.videoParam->mfx.IdrInterval = 1;
+        }
+        else if (pv->param.videoParam->mfx.CodecId == MFX_CODEC_AVC)
+        {
+            pv->param.videoParam->mfx.IdrInterval = 0;
+        }
+        pv->param.videoParam->AsyncDepth = 60;
+    }
     // sanitize some settings that affect memory consumption
     if (!pv->is_sys_mem)
     {
@@ -1589,7 +1652,6 @@ int encqsvInit(hb_work_object_t *w, hb_job_t *job)
         // LookAheadDepth 10 will cause a hang with some driver versions
         pv->param.codingOption2.LookAheadDepth = FFMAX(pv->param.codingOption2.LookAheadDepth, 11);
     }
-
     /*
      * We may need to adjust GopRefDist, GopPicSize and
      * NumRefFrame to enable or disable B-pyramid, so do it last.
@@ -1605,15 +1667,18 @@ int encqsvInit(hb_work_object_t *w, hb_job_t *job)
     int err;
     mfxStatus sts;
     mfxVersion version;
+    mfxLoader loader;
     mfxVideoParam videoParam;
-    mfxExtBuffer *extParamArray[3];
+    mfxExtBuffer *extParamArray[5];
     mfxSession session = (mfxSession)0;
     mfxExtCodingOption  option1_buf, *option1 = &option1_buf;
     mfxExtCodingOption2 option2_buf, *option2 = &option2_buf;
     mfxExtCodingOptionSPSPPS sps_pps_buf, *sps_pps = &sps_pps_buf;
+    mfxExtAV1BitstreamParam av1_bitstream_buf, *av1_bitstream = &av1_bitstream_buf;
+    mfxExtHyperModeParam hyper_encode_buf, *hyper_encode = &hyper_encode_buf;
     version.Major = HB_QSV_MINVERSION_MAJOR;
     version.Minor = HB_QSV_MINVERSION_MINOR;
-    sts = MFXInit(pv->qsv_info->implementation, &version, &session);
+    sts = hb_qsv_create_mfx_session(pv->qsv_info->implementation, job->qsv.ctx->dx_index, &version, &session, &loader);
     if (sts != MFX_ERR_NONE)
     {
         hb_error("encqsvInit: MFXInit failed (%d) with implementation %d", sts, pv->qsv_info->implementation);
@@ -1651,7 +1716,7 @@ int encqsvInit(hb_work_object_t *w, hb_job_t *job)
         return -1;
     }
 #endif
-
+    /* MFXVideoENCODE_Init with desired encoding parameters */
     sts = MFXVideoENCODE_Init(session, pv->param.videoParam);
 // workaround for the early 15.33.x driver, should be removed later
 #define HB_DRIVER_FIX_33
@@ -1679,6 +1744,7 @@ int encqsvInit(hb_work_object_t *w, hb_job_t *job)
         MFXClose(session);
         return -1;
     }
+    /* Prepare the structures for query */
     memset(&videoParam, 0, sizeof(mfxVideoParam));
     videoParam.ExtParam = extParamArray;
     videoParam.NumExtParam = 0;
@@ -1712,6 +1778,22 @@ int encqsvInit(hb_work_object_t *w, hb_job_t *job)
     {
         videoParam.ExtParam[videoParam.NumExtParam++] = (mfxExtBuffer*)option2;
     }
+    // introduced in API 2.5
+    memset(av1_bitstream, 0, sizeof(mfxExtAV1BitstreamParam));
+    av1_bitstream->Header.BufferId = MFX_EXTBUFF_AV1_BITSTREAM_PARAM;
+    av1_bitstream->Header.BufferSz = sizeof(mfxExtAV1BitstreamParam);
+    if (pv->qsv_info->capabilities & HB_QSV_CAP_AV1_BITSTREAM)
+    {
+        videoParam.ExtParam[videoParam.NumExtParam++] = (mfxExtBuffer*)av1_bitstream;
+    }
+    memset(hyper_encode, 0, sizeof(mfxExtHyperModeParam));
+    hyper_encode->Header.BufferId = MFX_EXTBUFF_HYPER_MODE_PARAM;
+    hyper_encode->Header.BufferSz = sizeof(mfxExtHyperModeParam);
+    if (pv->qsv_info->capabilities & HB_QSV_CAP_HYPERENCODE)
+    {
+        videoParam.ExtParam[videoParam.NumExtParam++] = (mfxExtBuffer*)hyper_encode;
+    }
+    /* Query actual encoding parameters after MFXVideoENCODE_Init, some of them could be overridden */
     sts = MFXVideoENCODE_GetVideoParam(session, &videoParam);
     if (sts != MFX_ERR_NONE)
     {
@@ -1736,7 +1818,7 @@ int encqsvInit(hb_work_object_t *w, hb_job_t *job)
     }
     else if (videoParam.mfx.CodecId == MFX_CODEC_HEVC)
     {
-        if (qsv_hevc_make_header(w, session) < 0)
+        if (qsv_hevc_make_header(w, session, &videoParam) < 0)
         {
             hb_error("encqsvInit: qsv_hevc_make_header failed");
 #if !HB_QSV_ONEVPL
@@ -1782,6 +1864,7 @@ int encqsvInit(hb_work_object_t *w, hb_job_t *job)
         {
             case MFX_CODEC_AVC:
             case MFX_CODEC_HEVC:
+            case MFX_CODEC_AV1:
                 pv->init_delay = &w->config->init_delay;
                 break;
             default: // unreachable
@@ -1801,10 +1884,10 @@ int encqsvInit(hb_work_object_t *w, hb_job_t *job)
         hb_error("encqsvInit: log_encoder_params failed (%d)", err);
         return -1;
     }
-    pv->max_buffer_size = videoParam.mfx.BufferSizeInKB;
+    pv->param.videoParam->mfx = videoParam.mfx;
     // AsyncDepth has now been set and/or modified by Media SDK
     // fall back to default if zero
-    pv->max_async_depth = videoParam.AsyncDepth ? videoParam.AsyncDepth : HB_QSV_ASYNC_DEPTH_DEFAULT;
+    pv->max_async_depth = videoParam.AsyncDepth ? videoParam.AsyncDepth : hb_qsv_param_default_async_depth();
     pv->async_depth     = 0;
 
     return 0;
@@ -1887,10 +1970,6 @@ void encqsvClose(hb_work_object_t *w)
             /* the list is already empty */
             hb_list_close(&pv->delayed_processing);
         }
-        if (pv->sws_context_to_nv12 != NULL)
-        {
-            sws_freeContext(pv->sws_context_to_nv12);
-        }
         if (pv->list_dts != NULL)
         {
             int64_t *item;
@@ -2308,7 +2387,7 @@ int encqsvWork(hb_work_object_t *w, hb_buffer_t **buf_in, hb_buffer_t **buf_out)
         }
 
         surface = qsv_enc_space->p_surfaces[surface_index];
-        qsv_yuv420_to_nv12(pv->sws_context_to_nv12, surface, in);
+        qsv_copy_buffer_to_surface(surface, in);
     }
     else
     {
