@@ -411,13 +411,13 @@ int         hb_video_quality_is_supported(uint32_t codec);
 int         hb_video_twopass_is_supported(uint32_t codec);
 
 int                hb_video_encoder_is_supported(int encoder);
-int                hb_video_encoder_pix_fmt_is_supported(int encoder, int pix_fmt);
+int                hb_video_encoder_pix_fmt_is_supported(int encoder, int pix_fmt, const char *profile);
 int                hb_video_encoder_get_depth   (int encoder);
 const char* const* hb_video_encoder_get_presets (int encoder);
 const char* const* hb_video_encoder_get_tunes   (int encoder);
 const char* const* hb_video_encoder_get_profiles(int encoder);
 const char* const* hb_video_encoder_get_levels  (int encoder);
-const int*         hb_video_encoder_get_pix_fmts(int encoder);
+const int*         hb_video_encoder_get_pix_fmts(int encoder, const char *profile);
 
 void  hb_audio_quality_get_limits(uint32_t codec, float *low, float *high, float *granularity, int *direction);
 float hb_audio_quality_get_best(uint32_t codec, float quality);
@@ -554,42 +554,66 @@ struct hb_job_s
          cfr:               0 (vfr), 1 (cfr), 2 (pfr) [see render.c]
          pass:              0, 1 or 2 (or -1 for scan)
          areBframes:        boolean to note if b-frames are used */
-#define HB_VCODEC_MASK         0x7FFFFFF
-#define HB_VCODEC_INVALID      0x0000000
-#define HB_VCODEC_THEORA       0x0000002
-#define HB_VCODEC_FFMPEG_MPEG4 0x0000010
-#define HB_VCODEC_FFMPEG_MPEG2 0x0000020
-#define HB_VCODEC_FFMPEG_VP8   0x0000040
-#define HB_VCODEC_FFMPEG_VP9   0x0000080
-#define HB_VCODEC_FFMPEG_VCE_H264 0x00040000
-#define HB_VCODEC_FFMPEG_VCE_H265 0x00080000
-#define HB_VCODEC_FFMPEG_NVENC_H264 0x00100000
-#define HB_VCODEC_FFMPEG_NVENC_H265 0x00200000
-#define HB_VCODEC_VT_H264       0x00400000
-#define HB_VCODEC_VT_H265       0x00800000
-#define HB_VCODEC_VT_H265_10BIT 0x01000000
-#define HB_VCODEC_VT_MASK       0x1C00000
-#define HB_VCODEC_FFMPEG_MF_H264 0x02000000
-#define HB_VCODEC_FFMPEG_MF_H265 0x04000000
-#define HB_VCODEC_FFMPEG_MASK  (0x00000F0|HB_VCODEC_FFMPEG_VCE_H264|HB_VCODEC_FFMPEG_VCE_H265|HB_VCODEC_FFMPEG_NVENC_H264|HB_VCODEC_FFMPEG_NVENC_H265|HB_VCODEC_FFMPEG_MF_H264|HB_VCODEC_FFMPEG_MF_H265)
-#define HB_VCODEC_QSV_H264     0x0000100
-#define HB_VCODEC_QSV_H265_8BIT     0x0000200
-#define HB_VCODEC_QSV_H265_10BIT    0x0000400
-#define HB_VCODEC_QSV_H265_MASK     0x0000600
-#define HB_VCODEC_QSV_H265     HB_VCODEC_QSV_H265_8BIT
-#define HB_VCODEC_QSV_MASK     0x0000F00
-#define HB_VCODEC_X264_8BIT    0x0010000
-#define HB_VCODEC_X264         HB_VCODEC_X264_8BIT
-#define HB_VCODEC_X264_10BIT   0x0020000
-#define HB_VCODEC_X264_MASK    0x0030000
-#define HB_VCODEC_H264_MASK    (HB_VCODEC_X264_MASK|HB_VCODEC_QSV_H264|HB_VCODEC_FFMPEG_VCE_H264|HB_VCODEC_FFMPEG_NVENC_H264|HB_VCODEC_VT_H264|HB_VCODEC_FFMPEG_MF_H264)
-#define HB_VCODEC_X265_8BIT    0x0001000
-#define HB_VCODEC_X265         HB_VCODEC_X265_8BIT
-#define HB_VCODEC_X265_10BIT   0x0002000
-#define HB_VCODEC_X265_12BIT   0x0004000
-#define HB_VCODEC_X265_16BIT   0x0008000
-#define HB_VCODEC_X265_MASK    0x000F000
-#define HB_VCODEC_H265_MASK    (HB_VCODEC_X265_MASK|HB_VCODEC_QSV_H265_MASK|HB_VCODEC_FFMPEG_VCE_H265|HB_VCODEC_FFMPEG_NVENC_H265|HB_VCODEC_FFMPEG_VT_H265|HB_VCODEC_FFMPEG_VT_H265_10BIT|HB_VCODEC_FFMPEG_MF_H265)
+#define HB_VCODEC_INVALID            0x00000000
+
+#define HB_VCODEC_AV1_MASK           0x40000000
+#define HB_VCODEC_H264_MASK          0x20000000
+#define HB_VCODEC_H265_MASK          0x10000000
+
+#define HB_VCODEC_SVT_AV1_MASK      (0x00800000 | HB_VCODEC_AV1_MASK | HB_VCODEC_FFMPEG_MASK)
+#define HB_VCODEC_X264_MASK         (0x00400000 | HB_VCODEC_H264_MASK)
+#define HB_VCODEC_X265_MASK         (0x00200000 | HB_VCODEC_H265_MASK)
+
+#define HB_VCODEC_VT_MASK            0x00080000
+#define HB_VCODEC_QSV_MASK           0x00040000
+#define HB_VCODEC_FFMPEG_MASK        0x00010000
+
+#define HB_VCODEC_THEORA             0x00000001
+
+#define HB_VCODEC_X264_8BIT         (0x00000002 | HB_VCODEC_X264_MASK)
+#define HB_VCODEC_X264              HB_VCODEC_X264_8BIT
+#define HB_VCODEC_X264_10BIT        (0x00000003 | HB_VCODEC_X264_MASK)
+
+#define HB_VCODEC_X265_8BIT         (0x00000004 | HB_VCODEC_X265_MASK)
+#define HB_VCODEC_X265              HB_VCODEC_X265_8BIT
+#define HB_VCODEC_X265_10BIT        (0x00000005 | HB_VCODEC_X265_MASK)
+#define HB_VCODEC_X265_12BIT        (0x00000006 | HB_VCODEC_X265_MASK)
+#define HB_VCODEC_X265_16BIT        (0x00000007 | HB_VCODEC_X265_MASK)
+
+#define HB_VCODEC_FFMPEG_MPEG4      (0x00000008 | HB_VCODEC_FFMPEG_MASK)
+#define HB_VCODEC_FFMPEG_MPEG2      (0x00000009 | HB_VCODEC_FFMPEG_MASK)
+
+#define HB_VCODEC_FFMPEG_VP8        (0x0000000A | HB_VCODEC_FFMPEG_MASK)
+#define HB_VCODEC_FFMPEG_VP9        (0x0000000B | HB_VCODEC_FFMPEG_MASK)
+#define HB_VCODEC_FFMPEG_VP9_10BIT  (0x0000000C | HB_VCODEC_FFMPEG_MASK)
+
+#define HB_VCODEC_FFMPEG_VCE_H264           (0x0000000D | HB_VCODEC_FFMPEG_MASK | HB_VCODEC_H264_MASK)
+#define HB_VCODEC_FFMPEG_VCE_H265           (0x0000000E | HB_VCODEC_FFMPEG_MASK | HB_VCODEC_H265_MASK)
+#define HB_VCODEC_FFMPEG_VCE_H265_10BIT     (0x0000000F | HB_VCODEC_FFMPEG_MASK | HB_VCODEC_H265_MASK)
+
+#define HB_VCODEC_FFMPEG_MF_H264    (0x00000020 | HB_VCODEC_FFMPEG_MASK | HB_VCODEC_H264_MASK)
+#define HB_VCODEC_FFMPEG_MF_H265    (0x00000021 | HB_VCODEC_FFMPEG_MASK | HB_VCODEC_H265_MASK)
+
+#define HB_VCODEC_FFMPEG_NVENC_H264         (0x00000030 | HB_VCODEC_FFMPEG_MASK | HB_VCODEC_H264_MASK)
+#define HB_VCODEC_FFMPEG_NVENC_H265         (0x00000031 | HB_VCODEC_FFMPEG_MASK | HB_VCODEC_H265_MASK)
+#define HB_VCODEC_FFMPEG_NVENC_H265_10BIT   (0x00000032 | HB_VCODEC_FFMPEG_MASK | HB_VCODEC_H265_MASK)
+
+#define HB_VCODEC_FFMPEG_SVT_AV1_8BIT       (0x00000041 | HB_VCODEC_SVT_AV1_MASK)
+#define HB_VCODEC_FFMPEG_SVT_AV1            HB_VCODEC_FFMPEG_SVT_AV1_8BIT
+#define HB_VCODEC_FFMPEG_SVT_AV1_10BIT      (0x00000042 | HB_VCODEC_SVT_AV1_MASK)
+
+#define HB_VCODEC_VT_H264           (0x00000050 | HB_VCODEC_VT_MASK | HB_VCODEC_H264_MASK)
+#define HB_VCODEC_VT_H265           (0x00000051 | HB_VCODEC_VT_MASK | HB_VCODEC_H265_MASK)
+#define HB_VCODEC_VT_H265_10BIT     (0x00000052 | HB_VCODEC_VT_MASK | HB_VCODEC_H265_MASK)
+
+#define HB_VCODEC_QSV_H264          (0x00000060 | HB_VCODEC_QSV_MASK | HB_VCODEC_H264_MASK)
+#define HB_VCODEC_QSV_H265_8BIT     (0x00000061 | HB_VCODEC_QSV_MASK | HB_VCODEC_H265_MASK)
+#define HB_VCODEC_QSV_H265_10BIT    (0x00000062 | HB_VCODEC_QSV_MASK | HB_VCODEC_H265_MASK)
+#define HB_VCODEC_QSV_H265          HB_VCODEC_QSV_H265_8BIT
+
+#define HB_VCODEC_QSV_AV1_8BIT      (0x00000070 | HB_VCODEC_QSV_MASK | HB_VCODEC_AV1_MASK)
+#define HB_VCODEC_QSV_AV1_10BIT     (0x00000071 | HB_VCODEC_QSV_MASK | HB_VCODEC_AV1_MASK)
+#define HB_VCODEC_QSV_AV1           HB_VCODEC_QSV_AV1_8BIT
 
 /* define an invalid CQ value compatible with all CQ-capable codecs */
 #define HB_INVALID_VIDEO_QUALITY (-1000.)
@@ -750,8 +774,6 @@ struct hb_job_s
     uint32_t        frames_to_skip;     // decode but discard this many frames
                                         //  initially (for frame accurate positioning
                                         //  to non-I frames).
-    PRIVATE int use_decomb;
-    PRIVATE int use_detelecine;
 
     // QSV-specific settings
     struct
@@ -773,6 +795,8 @@ struct hb_job_s
         } enc_info;
     } qsv;
 
+    int hw_decode;
+
 #ifdef __LIBHB__
     /* Internal data */
     hb_handle_t   * h;
@@ -798,6 +822,12 @@ struct hb_job_s
                                        // Other pipeline stages need to know
                                        // this.  E.g. sync and decsrtsub
 #endif
+#if HB_PROJECT_FEATURE_NVENC
+    struct
+    {
+        void *hw_device_ctx;
+    } nv_hw_ctx;
+#endif
 };
 
 /* Audio starts here */
@@ -826,7 +856,7 @@ struct hb_job_s
 #define HB_ACODEC_OPUS      0x04000000
 #define HB_ACODEC_FF_MASK   0x0FFF2800
 #define HB_ACODEC_PASS_FLAG 0x40000000
-#define HB_ACODEC_PASS_MASK   (HB_ACODEC_AC3 | HB_ACODEC_DCA | HB_ACODEC_DCA_HD | HB_ACODEC_FFAAC | HB_ACODEC_FFEAC3 | HB_ACODEC_FFFLAC | HB_ACODEC_MP2 | HB_ACODEC_MP3 | HB_ACODEC_FFTRUEHD)
+#define HB_ACODEC_PASS_MASK   (HB_ACODEC_AC3 | HB_ACODEC_DCA | HB_ACODEC_DCA_HD | HB_ACODEC_FFAAC | HB_ACODEC_FFEAC3 | HB_ACODEC_FFFLAC | HB_ACODEC_MP2 | HB_ACODEC_MP3 | HB_ACODEC_FFTRUEHD | HB_ACODEC_OPUS)
 #define HB_ACODEC_AUTO_PASS   (HB_ACODEC_PASS_FLAG | HB_ACODEC_PASS_MASK)
 #define HB_ACODEC_ANY         (HB_ACODEC_PASS_FLAG | HB_ACODEC_MASK)
 #define HB_ACODEC_AAC_PASS    (HB_ACODEC_PASS_FLAG | HB_ACODEC_FFAAC)
@@ -838,6 +868,7 @@ struct hb_job_s
 #define HB_ACODEC_MP2_PASS    (HB_ACODEC_PASS_FLAG | HB_ACODEC_MP2)
 #define HB_ACODEC_MP3_PASS    (HB_ACODEC_PASS_FLAG | HB_ACODEC_MP3)
 #define HB_ACODEC_TRUEHD_PASS (HB_ACODEC_PASS_FLAG | HB_ACODEC_FFTRUEHD)
+#define HB_ACODEC_OPUS_PASS   (HB_ACODEC_PASS_FLAG | HB_ACODEC_OPUS)
 
 #define HB_SUBSTREAM_BD_TRUEHD  0x72
 #define HB_SUBSTREAM_BD_AC3     0x76
@@ -1103,7 +1134,6 @@ struct hb_metadata_s
     hb_list_t * list_coverart;
 };
 
-// Update win/CS/HandBrake.Interop/HandBrakeInterop/HbLib/hb_title_s.cs when changing this struct
 struct hb_title_s
 {
     enum { HB_DVD_TYPE, HB_BD_TYPE, HB_STREAM_TYPE, HB_FF_STREAM_TYPE } type;
@@ -1139,6 +1169,8 @@ struct hb_title_s
     hb_content_light_metadata_t     coll;
     hb_rational_t   vrate;
     int             crop[4];
+    int             loose_crop[4];
+
     enum {HB_DVD_DEMUXER, HB_TS_DEMUXER, HB_PS_DEMUXER, HB_NULL_DEMUXER} demuxer;
     int             detected_interlacing;
     int             pcr_pid;                /* PCR PID for TS streams */
@@ -1156,6 +1188,7 @@ struct hb_title_s
     int           video_decode_support;
 #define HB_DECODE_SUPPORT_SW    0x01 // software (libavcodec or mpeg2dec)
 #define HB_DECODE_SUPPORT_QSV   0x02 // Intel Quick Sync Video
+#define HB_DECODE_SUPPORT_NVDEC 0x04 // Nvidia Nvdec
 
     hb_metadata_t * metadata;
 
@@ -1356,6 +1389,12 @@ typedef struct hb_filter_init_s
     int             cfr;
     int             grayscale;
     hb_rational_t   time_base;
+#if HB_PROJECT_FEATURE_NVENC
+    struct
+    {
+        void *hw_frames_ctx;
+    } nv_hw_ctx;
+#endif
 } hb_filter_init_t;
 
 typedef struct hb_filter_info_s
@@ -1418,7 +1457,8 @@ enum
     HB_FILTER_DETELECINE,
     HB_FILTER_COMB_DETECT,
     HB_FILTER_DECOMB,
-    HB_FILTER_DEINTERLACE,
+    HB_FILTER_YADIF,
+    HB_FILTER_BWDIF,
     HB_FILTER_VFR,
     // Filters that must operate on the original source image are next
     HB_FILTER_DEBLOCK,
@@ -1505,6 +1545,7 @@ int hb_output_color_transfer(hb_job_t * job);
 int hb_output_color_matrix(hb_job_t * job);
 
 int hb_get_bit_depth(int format);
+int hb_get_chroma_sub_sample(int format, int *h_shift, int *v_shift);
 int hb_get_best_pix_fmt(hb_job_t * job);
 
 #define HB_NEG_FLOAT_REG "(([-])?(([0-9]+([.,][0-9]+)?)|([.,][0-9]+))"
